@@ -56,7 +56,14 @@ class ChannelFlowControlBase:
                 self._pause_future.set_result(None)
                 self._pause_future = None
                 return
-            raise RuntimeError(f"Reader already resumed for {ip_address} ({id_})")
+            # Already resumed - this is idempotent, no error needed
+            if self._debug:
+                _LOGGER.debug(
+                    "Reader already resumed for %s (%s), ignoring",
+                    ip_address,
+                    id_,
+                )
+            return
 
         if self._pause_future is None or self._pause_future.done():
             if self._debug:
@@ -64,7 +71,13 @@ class ChannelFlowControlBase:
             self._pause_future = self._loop.create_future()
             return
 
-        raise RuntimeError(f"Reader already paused for {ip_address} ({id_})")
+        # Already paused - this is idempotent, no error needed
+        if self._debug:
+            _LOGGER.debug(
+                "Reader already paused for %s (%s), ignoring",
+                ip_address,
+                id_,
+            )
 
 
 class MultiplexerChannel:
@@ -188,8 +201,10 @@ class MultiplexerChannel:
             return
         pause_reader = self._local_output_under_water or self._remote_input_under_water
         if self._reader_paused != pause_reader:
-            self._reader_paused = pause_reader
+            # Call the callback first, then update state if successful
+            # This ensures state consistency even if callback fails
             self._pause_resume_reader_callback(pause_reader)
+            self._reader_paused = pause_reader
 
     @property
     def id(self) -> MultiplexerChannelId:
